package com.logicbig.example;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.Resource;
import org.springframework.hateoas.Resources;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RestController;
import java.util.List;
import java.util.stream.Collectors;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

@RestController
public class DeptController {
    @Autowired
    private EmployeeService employeeService;

    @GetMapping("/depts/{deptId}")
    public Resource<Dept> getDept(@PathVariable long deptId) {
        Dept dept = employeeService.getDeptByDeptId(deptId);
        Resource<Dept> deptResource = createDeptResource(dept);
        List<Employee> employees = employeeService.getEmployeesByDept(deptId);
        employees.stream()
                 .forEach(e -> deptResource.add(TaskController.createEmployeeLink(e.getEmployeeId())));
        deptResource.add(linkTo(methodOn(DeptController.class).getAllDepts()).withRel("depts"));
        return deptResource;
    }

    @GetMapping("/depts")
    public Resources<Resource<Dept>> getAllDepts() {
        List<Dept> depts = employeeService.getAllDept();
        Link selfRel = linkTo(methodOn(DeptController.class).getAllDepts()).withSelfRel();
        List<Resource<Dept>> deptResources = depts.stream()
                                                  .map(d -> createDeptResource(d))
                                                  .collect(Collectors.toList());
        return new Resources<>(deptResources, selfRel);
    }

    private static Resource<Dept> createDeptResource(Dept dept) {
        Link selfRel = linkTo(methodOn(DeptController.class).getDept(dept.getDeptId())).withSelfRel();
        return new Resource<>(dept, selfRel);
    }
}