package com.logicbig.example;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.Resource;
import org.springframework.hateoas.Resources;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import java.util.List;
import java.util.stream.Collectors;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;

@RestController
@RequestMapping("/employees")
public class EmployeeController {
    @Autowired
    private EmployeeService employeeService;

    @GetMapping
    public Resources<Resource<Employee>> getEmployees() {
        List<Employee> employeeList = employeeService.getAllEmployees();
        List<Resource<Employee>> employeeResources =
                employeeList.stream()
                            .map(EmployeeController::createEmployeeResource)
                            .collect(Collectors.toList());
        Link selfRel = linkTo(methodOn(EmployeeController.class)
                .getEmployees()).withSelfRel();
        return new Resources<>(employeeResources, selfRel);
    }

    @GetMapping("/{employeeId}")
    public Resource<Employee> getEmployeeById(@PathVariable long employeeId) {
        Employee employee = employeeService.getEmployeeById(employeeId);
        Link selfLink = linkTo(methodOn(EmployeeController.class)
                .getEmployeeById(employeeId)).withSelfRel();
        return new Resource<>(employee, selfLink, createTasksLink(employeeId),
                createDeptLink(employee.getDept().getDeptId()));
    }

    private static Resource<Employee> createEmployeeResource(Employee e) {
        Link employeeLink = linkTo(methodOn(EmployeeController.class)
                .getEmployeeById(e.getEmployeeId())).withSelfRel();
        return new Resource<>(e, employeeLink);
    }

    private static Link createTasksLink(long employeeId) {
        return linkTo(methodOn(TaskController.class)
                .getEmployeeTasks(employeeId)).withRel("tasks");
    }

    private Link createDeptLink(long deptId) {
        return linkTo(methodOn(DeptController.class)
                .getDept(deptId)).withRel("dept");
    }
}