package com.logicbig.example;

import org.springframework.stereotype.Service;
import java.util.List;
import java.util.stream.Collectors;

public interface EmployeeService {

    List<Employee> getAllEmployees();

    Employee getEmployeeById(long employeeId);

    List<Task> getTasksByEmployeeId(long employeeId);

    Task getTaskByTaskId(long taskId);

    List<Task> getAllTasks();

    List<Dept> getAllDept();

    List<Employee> getEmployeesByDept(long deptId);

    Dept getDeptByDeptId(long deptId);

    @Service
    public static class DefaultEmployeeService implements EmployeeService {
        private final List<Employee> employees;
        private final List<Task> tasks;
        private final List<Dept> depts;

        public DefaultEmployeeService() {
            Dept qa = Dept.create(1, "QA");
            Dept it = Dept.create(2, "IT");
            Employee lara = Employee.create(1, "Lara", qa);
            Employee tom = Employee.create(2, "Tom", qa);
            Employee tina = Employee.create(3, "Tina", it);
            Employee john = Employee.create(4, "John", it);
            this.employees = List.of(lara, tina, tom, john);
            Task development = Task.create(1, "Development", john);
            Task designing = Task.create(2, "Designing", tina);
            Task testing = Task.create(3, "Testing", lara);
            Task documentation = Task.create(4, "Documentation", tina);
            Task deployment = Task.create(5, "Deployment", john);
            this.tasks = List.of(development, designing, testing, documentation, deployment);
            this.depts = List.of(qa, it);
        }

        @Override
        public List<Employee> getAllEmployees() {
            return employees;
        }

        @Override
        public Employee getEmployeeById(long employeeId) {
            return employees.stream()
                            .filter(e -> e.getEmployeeId() == employeeId)
                            .findFirst()
                            .orElseThrow();
        }

        @Override
        public List<Task> getTasksByEmployeeId(long employeeId) {
            return tasks.stream()
                        .filter(t -> t.getEmployee().getEmployeeId() == employeeId)
                        .collect(Collectors.toList());
        }

        @Override
        public Task getTaskByTaskId(long taskId) {
            return tasks.stream()
                        .filter(t -> t.getTaskId() == taskId)
                        .findFirst()
                        .orElseThrow();
        }

        @Override
        public List<Task> getAllTasks() {
            return tasks;
        }

        @Override
        public List<Dept> getAllDept() {
            return depts;
        }

        @Override
        public List<Employee> getEmployeesByDept(long deptId) {
            return getAllEmployees().stream()
                                    .filter(e -> deptId == e.getDept().getDeptId())
                                    .collect(Collectors.toList());
        }

        @Override
        public Dept getDeptByDeptId(long deptId) {
            return depts.stream()
                        .filter(d -> d.getDeptId() == deptId)
                        .findAny()
                        .orElseThrow();
        }
    }
}